'use strict'

const fs = require('fs-extra')
const _ = require('lodash');
const http = require('http');
const fortune = require('fortune');
const jsonApi = require('fortune-json-api');
const nedbAdapter = require('fortune-nedb-uuid');
const path = require('path');

class JamServer {
  constructor(settings) {
    this.settings = settings;
    this.settings.factoriesPath = path.normalize(path.join(this.settings.factoriesPath));
    this.settings.dataPath = path.normalize(path.join(this.settings.dataPath));
  }
  createConfigFile() {
    fs.outputJson(path.join(process.cwd(), 'jamconfig.json'), this.settings, function(err){
      if(err) {
        console.error(err);
      }
    });
  }
  startWebServer() {
    this.settings.factoriesPath = (path.isAbsolute(this.settings.factoriesPath)) ? path.normalize(path.join(this.settings.factoriesPath)) : path.normalize(path.join(process.cwd(), this.settings.factoriesPath));
    this.settings.dataPath = (path.isAbsolute(this.settings.dataPath)) ? path.normalize(path.join(this.settings.dataPath)) : path.normalize(path.join(process.cwd(), this.settings.dataPath));
    this.factories = this.loadFactories();
    this.initializeServer();
  }

  loadFactories() {
    let factories = {};
    return new Promise((resolve,reject) => {
      console.log(this.settings.factoriesPath);
      fs.readdir(this.settings.factoriesPath, (err, data) => {
        if(err){
          reject(err);
        }else{
          if(!data.length){
            reject(new Error('The factories directory was found, but no factories were found inside.'));
          } else {
            for (var i = 0; i < data.length; i++) {
              let filepath = path.join(this.settings.factoriesPath, data[i]);
              let resourceName = path.basename(data[i],'.js');
              factories[resourceName] = require(filepath);
            }
            resolve(factories);
          }
        }
      });
    });
  }
  initializeServer() {
    this.store = fortune({
      adapter: {
        type: nedbAdapter,
        options: {
          dbPath: this.settings.dataPath
        }
      },
      serializers: [{
        type: jsonApi,
        options: {prefix: ''}
      }]
    });
    this.factories.then((factoryResources, err) => {
      if(!err) {
        for (var factory in factoryResources) {
          this.store.defineType(factory, factoryResources[factory]);
        }
        this.server = http.createServer(fortune.net.http(this.store));
        this.store.connect().then(() => {
          this.server.listen(this.settings.portNumber);
          console.log('\n## JSON API Mocks Running ##');
          console.log(`Port Number: ${this.settings.portNumber}`);
          console.log(`Factory Path: ${this.settings.factoriesPath}`);
          console.log(`Data Path: ${this.settings.dataPath}`);
          console.log(`Resources Found: ` + _.keys(factoryResources));
        });
      } else {
        console.error(err.message);
      }
    }).catch((err) => {
      console.error(err.message);
    });
  }

};
module.exports = JamServer
